"use strict";
/**
 * Copyright (c) 2018-2022 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MoleculeTypeColorThemeProvider = exports.MoleculeTypeColorTheme = exports.moleculeTypeColor = exports.getMoleculeTypeColorThemeParams = exports.MoleculeTypeColorThemeParams = void 0;
var color_1 = require("../../mol-util/color");
var structure_1 = require("../../mol-model/structure");
var util_1 = require("../../mol-model/structure/util");
var param_definition_1 = require("../../mol-util/param-definition");
var legend_1 = require("../../mol-util/legend");
var DefaultMoleculeTypeColor = (0, color_1.Color)(0xffff99);
var Description = 'Assigns a color based on the molecule type of a residue.';
exports.MoleculeTypeColorThemeParams = {
    saturation: param_definition_1.ParamDefinition.Numeric(0, { min: -6, max: 6, step: 0.1 }),
    lightness: param_definition_1.ParamDefinition.Numeric(0, { min: -6, max: 6, step: 0.1 }),
    colors: param_definition_1.ParamDefinition.Group({
        water: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0x386cb0)),
        ion: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0xf0027f)),
        protein: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0xbeaed4)),
        rna: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0xfdc086)),
        dna: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0xbf5b17)),
        pna: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0x42A49A)),
        saccharide: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0x7fc97f)),
        lipid: param_definition_1.ParamDefinition.Color((0, color_1.Color)(0xcccccc)),
    })
};
function getMoleculeTypeColorThemeParams(ctx) {
    return exports.MoleculeTypeColorThemeParams; // TODO return copy
}
exports.getMoleculeTypeColorThemeParams = getMoleculeTypeColorThemeParams;
function moleculeTypeColor(props, unit, element) {
    var c = DefaultMoleculeTypeColor;
    var moleculeType = (0, util_1.getElementMoleculeType)(unit, element);
    switch (moleculeType) {
        case 2 /* Water */:
            c = props.colors.water;
            break;
        case 3 /* Ion */:
            c = props.colors.ion;
            break;
        case 5 /* Protein */:
            c = props.colors.protein;
            break;
        case 6 /* RNA */:
            c = props.colors.rna;
            break;
        case 7 /* DNA */:
            c = props.colors.dna;
            break;
        case 8 /* PNA */:
            c = props.colors.pna;
            break;
        case 9 /* Saccharide */:
            c = props.colors.saccharide;
            break;
        case 4 /* Lipid */:
            c = props.colors.lipid;
            break;
    }
    c = color_1.Color.saturate(c, props.saturation);
    c = color_1.Color.darken(c, -props.lightness);
    return c;
}
exports.moleculeTypeColor = moleculeTypeColor;
function MoleculeTypeColorTheme(ctx, props) {
    function color(location) {
        if (structure_1.StructureElement.Location.is(location)) {
            return moleculeTypeColor(props, location.unit, location.element);
        }
        else if (structure_1.Bond.isLocation(location)) {
            return moleculeTypeColor(props, location.aUnit, location.aUnit.elements[location.aIndex]);
        }
        return DefaultMoleculeTypeColor;
    }
    return {
        factory: MoleculeTypeColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: (0, legend_1.TableLegend)(Object.keys(props.colors).map(function (name) {
            return [name, props.colors[name]];
        }).concat([['Other/unknown', DefaultMoleculeTypeColor]]))
    };
}
exports.MoleculeTypeColorTheme = MoleculeTypeColorTheme;
exports.MoleculeTypeColorThemeProvider = {
    name: 'molecule-type',
    label: 'Molecule Type',
    category: "Residue Property" /* Residue */,
    factory: MoleculeTypeColorTheme,
    getParams: getMoleculeTypeColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(exports.MoleculeTypeColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
//# sourceMappingURL=molecule-type.js.map