/**
 * Copyright (c) 2018-2022 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { Color } from '../../mol-util/color';
import { StructureElement, Bond } from '../../mol-model/structure';
import { getElementMoleculeType } from '../../mol-model/structure/util';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { TableLegend } from '../../mol-util/legend';
var DefaultMoleculeTypeColor = Color(0xffff99);
var Description = 'Assigns a color based on the molecule type of a residue.';
export var MoleculeTypeColorThemeParams = {
    saturation: PD.Numeric(0, { min: -6, max: 6, step: 0.1 }),
    lightness: PD.Numeric(0, { min: -6, max: 6, step: 0.1 }),
    colors: PD.Group({
        water: PD.Color(Color(0x386cb0)),
        ion: PD.Color(Color(0xf0027f)),
        protein: PD.Color(Color(0xbeaed4)),
        rna: PD.Color(Color(0xfdc086)),
        dna: PD.Color(Color(0xbf5b17)),
        pna: PD.Color(Color(0x42A49A)),
        saccharide: PD.Color(Color(0x7fc97f)),
        lipid: PD.Color(Color(0xcccccc)),
    })
};
export function getMoleculeTypeColorThemeParams(ctx) {
    return MoleculeTypeColorThemeParams; // TODO return copy
}
export function moleculeTypeColor(props, unit, element) {
    var c = DefaultMoleculeTypeColor;
    var moleculeType = getElementMoleculeType(unit, element);
    switch (moleculeType) {
        case 2 /* Water */:
            c = props.colors.water;
            break;
        case 3 /* Ion */:
            c = props.colors.ion;
            break;
        case 5 /* Protein */:
            c = props.colors.protein;
            break;
        case 6 /* RNA */:
            c = props.colors.rna;
            break;
        case 7 /* DNA */:
            c = props.colors.dna;
            break;
        case 8 /* PNA */:
            c = props.colors.pna;
            break;
        case 9 /* Saccharide */:
            c = props.colors.saccharide;
            break;
        case 4 /* Lipid */:
            c = props.colors.lipid;
            break;
    }
    c = Color.saturate(c, props.saturation);
    c = Color.darken(c, -props.lightness);
    return c;
}
export function MoleculeTypeColorTheme(ctx, props) {
    function color(location) {
        if (StructureElement.Location.is(location)) {
            return moleculeTypeColor(props, location.unit, location.element);
        }
        else if (Bond.isLocation(location)) {
            return moleculeTypeColor(props, location.aUnit, location.aUnit.elements[location.aIndex]);
        }
        return DefaultMoleculeTypeColor;
    }
    return {
        factory: MoleculeTypeColorTheme,
        granularity: 'group',
        color: color,
        props: props,
        description: Description,
        legend: TableLegend(Object.keys(props.colors).map(function (name) {
            return [name, props.colors[name]];
        }).concat([['Other/unknown', DefaultMoleculeTypeColor]]))
    };
}
export var MoleculeTypeColorThemeProvider = {
    name: 'molecule-type',
    label: 'Molecule Type',
    category: "Residue Property" /* Residue */,
    factory: MoleculeTypeColorTheme,
    getParams: getMoleculeTypeColorThemeParams,
    defaultValues: PD.getDefaultValues(MoleculeTypeColorThemeParams),
    isApplicable: function (ctx) { return !!ctx.structure; }
};
//# sourceMappingURL=molecule-type.js.map